/*
 Satellite Domain
*/

domain {

   types { Satellite, Direction, Instrument, Mode }

   world {
      predicate On_Board(Instrument, Satellite);
      predicate Supports(Instrument, Mode);
      predicate Calibration_Target(Instrument, Direction);      
   }

   state {
      function  Direction  Pointing(Satellite);
      predicate            Power_Available(Satellite);
      predicate            Power_On(Instrument);
      predicate            Calibrated(Instrument);
      predicate            Have_Image(Direction, Mode);
   }



  action Turn_To(Satellite s, Direction d_new, Direction d_prev){
       duration:
          uniform 50 100;
       conditions:
          @start: Pointing(s)  = d_prev;
          //@start: Pointing(s) != d_new;
       effects:
          @start: Pointing(s) = undefined;
          @end: Pointing(s) = d_new;
   }

  action Switch_On(Instrument i, Satellite s){
       duration:
          constant 10;
       conditions:
          @overall: On_Board(i, s);
          @start: Power_Available(s);
       effects:
          @start:  !Power_Available(s);
          @start:  !Calibrated(i);
          @end:    Power_On(i);
   }

  action Switch_Off(Instrument i, Satellite s){
       duration:
          constant 1;
       conditions:
          @overall: On_Board(i, s);
          @start: Power_On(i);
       effects:
          @start: !Power_On(i);
          @end:   Power_Available(s);
  }

  action Calibrate(Satellite s, Instrument i, Direction d){
        duration:
           uniform 40 70;
        conditions:
           @overall: On_Board(i,s);
           @start:   Calibration_Target(i,d);
           @overall: Power_On(i);
        effects:
           @end:     Calibrated(i);
  }

  action Take_Image(Satellite s, Direction d, Instrument i, Mode m){
        duration:
            uniform 70 100;
        conditions:
            @overall: Calibrated(i);
            @overall: On_Board(i,s);
            @overall: Supports(i,m);
            @overall: Power_On(i);
            @overall: Pointing(s)=d;
        effects:
            @end: Have_Image(d,m);
  }
}